
#ifndef SYMTAB_H_INCLUDED
#define SYMTAB_H_INCLUDED

/* *************************************************************** Strukturen */

/* anonyme Strukturen verhindern den Zugriff auf interne Daten von außerhalb */
typedef struct sSymtab symtab;

/**@brief Funktionsprototyp für eine Funktion, die Daten eines bestimmten Typs
 * freigibt.
 */
typedef void(*fDestructor)(void*);

/**@brief Struktur eines Symboltabelleneintrages.
 * 
 * Ein Symbol besteht aus einer Symbolklasse, einem Namen und damit assoziierten
 * Daten. Der Tabelle selbst ist die Struktur der Daten nicht bekannt.
 */
typedef struct
{
	int class;               /**< Symbolklasse */
	const char* id;          /**< Bezeichner */
	void* data;              /**< assoziierte Daten */
	
	fDestructor destructor;  /**< Funktion, die die assoziierten Daten freigibt */
} symtabSymbol;

/**@brief Funktionsprototyp einer Funktion, die einen Eintrag der Symboltabelle
 * ausgibt.
 */
typedef void(*fSymtabPrinter)(const char* id, const symtabSymbol* sym);

/* ***************************************************************** Methoden */

/**@brief Erstellt eine neue Instanz einer Symboltabelle und gibt den Zeiger
 * darauf zurück.
 * @note Die so erstellte Symboltabelle hat bereits eine erste
 * Deklarationsschicht.
 * 
 * @param hint  geschätzte Anzahl aller Symbole
 * @return Zeiger auf eine neue Symboltabelle
 */
extern symtab* newSymtab(unsigned int hint);

/**@brief Löscht eine Symboltabelle und alle mit ihr assoziierten Daten.
 * 
 * @param table   Zieltabelle
 */
extern void deleteSymtab(symtab* table);

/**@brief Fügt eine neue Deklarationsebene in die Tabelle ein.
 * 
 * Neue Deklarationsebenen sorgen dafür, dass Neudefinitionen bereits
 * vorhandener Symbole überdeckt werden. Wird die zusätzliche Schicht wieder
 * entfernt, ist die alte Definition erneut gültig (siehe symtabPop).
 * 
 * @param table   Zieltabelle
 */
extern void symtabPush(symtab* table);

/**@brief Entfernt eine Deklarationsebene aus der Symboltabelle.
 * @note Wenn die oberste Schicht gelöscht wird, kommt es zu einer Fehlermeldung
 * gefolgt von einem harten Abbruch des Programms (exit(-1)).
 * 
 * @param table   Zieltabelle
 */
extern void symtabPop(symtab* table);

/**@brief Erstellt ein neues Symbol, das in eine Symboltabelle eingetragen
 * werden kann.
 * 
 * @param class  Symbolklasse
 * @param id     Bezeichner des Symbols
 * @note Wird von der Funktion kopiert. Der Aufrufer ist für den Speicher
 * verantwortlich.
 * 
 * @return Zeiger auf das neue Symbol
 */
extern symtabSymbol* newSymtabSymbol(int class, const char* id);

/**@brief Löscht ein Symboltabellen Symbol.
 * @note Dadurch wird es nicht aus der Symboltabelle entfernt.
 */
extern void deleteSymtabSymbol(symtabSymbol* symb);

/**@brief Deklariert ein neues Symbol und assoziiert Daten damit.
 * 
 * @param table  Zieltabelle
 * @param id     Name des Symbols
 * @param sym    Symbol
 * @note Wird von der Funktion kopiert. Der Aufrufer muss den Speicher am Ende
 * unter Benutzung von deleteSymtabSymbol freigeben. Die assoziierten Daten
 * werden von Symboltabelle weiter verwaltet und dürfen nicht von außen
 * freigegeben werden.
 * 
 * @return 1, falls das Symbol noch nicht vorhanden war \n
 *         2, falls es schon vorhanden war, aber nicht in der aktuellen Schicht \n
 *         0 sonst
 */
extern int symtabDeclare(symtab* table, const symtabSymbol* sym);

/**@brief Gibt ein bereits deklariertes Symbol zurück.
 * 
 * @param table  Quelltabelle
 * @param id     Name des Symbols
 * 
 * @return Zeiger auf das Symbol, falls es existiert \n
 *         0 sonst
 */
extern const symtabSymbol* symtabGet(const symtab* table, const char* id);

/**@brief Gibt alle aktuell definierten Symbole der Symboltabelle aus.
 */
extern void symtabPrint(const symtab* table, fSymtabPrinter printer);

#endif
